
#include "sync.h"

#include <assert.h>
#include <stdlib.h>
#include <string.h>

#include <generator.h>
#include <netndfsm.h>

#if defined(__GNUC__) && !defined(__MINGW32__)
  #include <ucontext.h>
#elif defined(WIN32)
  #include <win32_ucontext.h>
#else
  #error "Unknown platform"
#endif

#ifdef SIGSTKSZ
#undef SIGSTKSZ
#endif
#define SIGSTKSZ        (8192*32)

static char stack[SIGSTKSZ];

static ucontext_t scenario_context;
static ucontext_t testbench_context;

static TestEngine *local_testEngine;

static bool initscenario = false;
static bool netscenario = false;

static int last_operation = 0;

static int parsed_argc;
static char **parsed_argv;

static void parse_parameters(char *str, const char *scen_name)
{
   	parsed_argc = 0;

   	parsed_argv = (char**)malloc((parsed_argc + 1) * sizeof(void*));
	parsed_argv[parsed_argc] = (char*)malloc((strlen(scen_name) + 1) * sizeof(char));
	strcpy(parsed_argv[parsed_argc], scen_name);
	parsed_argv[parsed_argc][strlen(scen_name)] = 0;
	parsed_argc++;

    if(str != NULL)
    {
        int i = 0;
        int start_pos = 0;
        int end_pos = 0;

        while(str[i] != 0)
        {
            bool commented = false;
            bool commented_quotes = false;

            // searching for the start of parameter
            while(str[i] == 0x20)
            {
                i++;
            }

            start_pos = i;

            // searching for the end of parameter
            while((str[i] != 0x20 || str[i] == 0x20 && commented) && str[i] != 0)
            {
                if(!commented_quotes)
                {
                    if(commented)
                    {
                        if(str[i] == '\\')
                            commented_quotes = true;
                    }

                    if(str[i] == '\'')
                        commented = ~commented;
                }
                else
                {
                    commented_quotes = false;
                }

                i++;
            }

            end_pos = i;

            // if we find something, it can be saved into argv with argc incrementation
            if(end_pos - start_pos > 0)
            {
                int j = 0;
                int k = 0;
                int size = 0;

                parsed_argv = (char**)realloc(parsed_argv, (parsed_argc + 1) * sizeof(void*));

                // computing size of symbols to copy (except screened letters)
                for(j = start_pos; j < end_pos; j++)
                {
                    if(commented_quotes)
                    {
                        size++;
                        commented_quotes = false;
                        continue;
                    }

                    if(str[j] == '\\')
                    {
                        commented_quotes = true;
                        continue;
                    }

                    if(str[j] == '\"' || str[j] == '\'')
                        continue;

                    size++;
                }
                size++;

                parsed_argv[parsed_argc] = (char*)malloc(size * sizeof(char));

                // copying from str to parsed_argv
                for(j = start_pos; j < end_pos; j++)
                {
                    if(commented_quotes)
                    {
                        parsed_argv[parsed_argc][k] = str[j];
                        k++;
                        commented_quotes = false;
                        continue;
                    }

                    if(str[j] == '\\')
                    {
                        commented_quotes = true;
                        continue;
                    }

                    if(str[j] == '\"' || str[j] == '\'')
                        continue;

                    parsed_argv[parsed_argc][k] = str[j];
                    k++;
                }

                parsed_argv[parsed_argc][size - 1] = 0;

                parsed_argc++;
            }
        }
    }
}

int start_scenario_from_function(void(*function)(void))
{
    getcontext(&scenario_context);
    scenario_context.uc_link = &testbench_context;
    scenario_context.uc_stack.ss_sp = stack;
    scenario_context.uc_stack.ss_size = SIGSTKSZ;
    makecontext(&scenario_context, function, 0);
    swapcontext(&testbench_context, &scenario_context);

    return 0;
}

int end_scenario(void)
{
    return 0;
}

int wait_for_apply(void)
{
    return last_operation;
}

// scenario
void wait_for_check(void)
{
    swapcontext(&scenario_context, &testbench_context);
}

// testbench
void apply_check(void)
{
    swapcontext(&testbench_context, &scenario_context);
}

void apply_finish(void)
{
    last_operation = ACTION_FINISH_OPERATION;
}

void apply_operation(void)
{
    last_operation = ACTION_START_OPERATION;
}

void end_call(void)
{
}

